#!/usr/bin/env python3
#
# Dieses Dokument ist in der Zusammenarbeit von Informatikreferendaren und
# ehemaligen Informatikreferendaren der Studienseminare (heute ZfsL) Arnsberg,
# Hamm und Solingen entstanden.
#
# Das Dokument steht unter der Lizenz: Creative Commons by-nc-sa Version 4.0
# http://creativecommons.org/licenses/by-nc-sa/4.0/deed.de
#
# Nach dieser Lizenz darf das Dokument beliebig kopiert und bearbeitet werden,
# sofern das Folgeprodukt wiederum unter gleichen Lizenzbedingungen vertrieben
# und auf die ursprünglichen Urheber verwiesen wird.
# Eine kommerzielle Nutzung ist ausdrücklich ausgeschlossen.
#
# Die Namensnennung durch einen Verweis und die Lizenzangabe der ursprünglichen
# Urheber auf den Materialien für Schülerinnen und Schüler ist erforderlich.
#
# Die Sammlung der Dokumente steht unter
# http://ddi.uni-wuppertal.de/material/materialsammlung/
# zur Verfügung.
#
# Weitere Lizenz: GPL V3.0
# Siehe http://www.gnu.org/licenses/gpl-3.0.html
#
# $Id: binarytree.py 670 2014-06-21 13:06:15Z pieper $

class BinaryTree (object): 
	"""
	Die Klasse BinaryTree
	Mithilfe der Klasse BinaryTree können beliebig viele Inhaltsobjekte in 
	einem Binärbaum verwaltet werden. 
	Ein Objekt der Klasse stellt entweder einen leeren Baum dar oder verwaltet 
	ein Inhaltsobjekt sowie einen linken und einen rechten Teilbaum, die 
	ebenfalls Objekte der Klasse BinaryTree sind.
	"""

	def __init__(self, pObject=None, pLeftTree=None, pRightTree=None):
		"""
		Nach dem Aufruf des Konstruktors ohne Parameterwerte existiert ein 
		leerer Binärbaum.
		Wenn der Parameter pObject ungleich null ist, existiert nach dem Aufruf 
		des Konstruktors der Binärbaum und hat pObject als Inhaltsobjekt und 
		zwei leere Teilbäume. Falls der Parameter null ist, wird ein leerer 
		Binärbaum erzeugt.
		Wenn der Parameter pObject ungleich null ist, wird ein Binärbaum mit 
		pObject als Inhaltsobjekt und den beiden Teilbäumen pLeftTree und 
		pRightTree erzeugt. Sind pLeftTree oder pRightTree gleich null, wird der 
		entsprechende Teilbaum als leerer Binärbaum eingefügt. 
		Wenn der Parameter pObject gleich null ist, wird ein leerer Binärbaum 
		erzeugt.
		"""
		self._inhalt= pObject
		if pObject!= None:
			self._linkerTeilbaum=  pLeftTree
			self._rechterTeilbaum= pRightTree

		
	def isEmpty(self):
		"""Diese Anfrage liefert den Wahrheitswert true, wenn der Binärbaum leer 
		ist, sonst liefert sie den Wert false."""
		return self._inhalt==None

	def setObject(self, pObject=None):
		"""Wenn der Binärbaum leer ist, wird der Parameter pObject als 
		Inhaltsobjekt sowie ein leerer linker und rechter Teilbaum eingefügt. 
		Ist der Binärbaum nicht leer, wird das Inhaltsobjekt durch pObject 
		ersetzt. Die Teilbäume werden nicht geändert. 
		Wenn pObject null ist, bleibt der Binärbaum unverändert."""
		if pObject == None:
			return
		elif self.isEmpty():
			self._linkerTeilbaum=  BinaryTree()
			self._rechterTeilbaum= BinaryTree()
		self._inhalt= pObject

	def getObject(self):
		"""Diese Anfrage liefert das Inhaltsobjekt des Binärbaums. 
		Wenn der Binärbaum leer ist, wird null zurückgegeben."""
		return self._inhalt

	def setLeftTree(self, pTree=None):
		"""Wenn der Binärbaum leer ist, wird pTree nicht angehängt. Andernfalls
		erhält der Binärbaum den übergebenen Baum als linken Teilbaum.
		Falls der Parameter null ist, ändert sich nichts."""
		if self.isEmpty() or pTree==None:
			return
		self._linkerTeilbaum= pTree  
		

	def setRightTree(self, pTree=None):
		"""Wenn der Binärbaum leer ist, wird pTree nicht angehängt. Andernfalls
		erhält der Binärbaum den übergebenen Baum als rechten Teilbaum.
		Falls der Parameter null ist, ändert sich nichts."""
		if self.isEmpty() or pTree==None:
			return
		self._rechterTeilbaum= pTree


	def getLeftTree(self):
		"""Diese Anfrage liefert den linken Teilbaum des Binärbaumes. Der 
		Binärbaum ändert sich nicht. Wenn der Binärbaum leer ist, wird null
		zurückgegeben."""
		if self.isEmpty():
			return None
		else:	
			return self._linkerTeilbaum


	def getRightTree(self):
		"""Diese Anfrage liefert den rechten Teilbaum des Binärbaumes. 
		Der Binärbaum ändert sich nicht. Wenn der Binärbaum leer ist, wird null
		zurückgegeben."""
		if self.isEmpty():
			return None
		else:	
			return self._rechterTeilbaum


	def _printBaumPre(self):
		"""preorder"""
		if debug: print(str(self._inhalt)+" ",end="")
		if self._linkerTeilbaum != None: self._linkerTeilbaum._printBaumPre()
		if self._rechterTeilbaum != None: self._rechterTeilbaum._printBaumPre()


	def _printBaumPost(self):
		"""postorder"""
		if self._linkerTeilbaum != None: self._linkerTeilbaum._printBaumPost()
		if self._rechterTeilbaum != None: self._rechterTeilbaum._printBaumPost()
		if debug: print(str(self._inhalt)+" ",end="")


	def _printBaumIn(self):
		"""inorder"""
		if self._linkerTeilbaum != None: self._linkerTeilbaum._printBaumIn()
		if debug: print(str(self._inhalt)+" ",end="")
		if self._rechterTeilbaum != None: self._rechterTeilbaum._printBaumIn()


	def _printBaumEingerueckt(self, ebene=0):
		if self._linkerTeilbaum != None:
			self._linkerTeilbaum._printBaumEingerueckt(ebene+1)
		print(' '*ebene + str(self._inhalt))
		if self._rechterTeilbaum != None: 
			self._rechterTeilbaum._printBaumEingerueckt(ebene+1)
			

	def __str__(self):	
		sys.stdout = x = ListStream()
		print("\nBinärbaum:\n"+10*"=")
		if self.isEmpty():
			print("ist leer")
		else:
			#self._printBaumIn()
			self._printBaumEingerueckt()
		sys.stdout = sys.__stdout__
		return str(x.data)
	
import sys
class ListStream:
    def __init__(self):
        self.data = ""
    def write(self, s):
        self.data+=s

if __name__ == "__main__":
	debug = True
	ueberschrift="Selbsttest der Klasse BinaryTree\n"
	print (ueberschrift+(ueberschrift.__len__()-1)*"*")
	
	print("baeumchenLeer= BinaryTree()")
	baeumchenLeer= 	BinaryTree()
	print (baeumchenLeer)
	
	print('baeumchenJ= BinaryTree("Joe")')
	baeumchenJ= BinaryTree("Joe")
	print (baeumchenJ)

	print('baeumchenM= BinaryTree("Marie")')
	baeumchenM= BinaryTree("Marie")
	print (baeumchenM)
	
	print('baumMitTeilBaeumchen= BinaryTree("Zappa", baeumchenM, baeumchenJ)')
	baumMitTeilBaeumchen= BinaryTree("Zappa", baeumchenM, baeumchenJ)
	print (baumMitTeilBaeumchen)
	
	print('baumKind= BinaryTree("Kind", BinaryTree("Mutter"), BinaryTree("Vatter"))')
	baumKind= BinaryTree("Kind", BinaryTree("Mutter"), BinaryTree("Vatter"))
	print(baumKind)

	print('baumMitKindern=BinaryTree("Scheffe", baumKind, baumMitTeilBaeumchen)')
	baumMitKindern=BinaryTree("Scheffe", baumKind, baumMitTeilBaeumchen)
	print (baumMitKindern)
	
	print('hdiaebfcg')
	baumD=BinaryTree("d", BinaryTree("h"), 	BinaryTree("i"))
	baumA=BinaryTree("a", baumD, 			BinaryTree("e"))
	baumC=BinaryTree("c", BinaryTree("f"), 	BinaryTree("g"))
	baumB=BinaryTree("b", baumA,            baumC          )
	print (baumB)

    #       I   m  L
    #           *                  
	#             Lm         K      
	#            | -               |
	#            |    K      m  S  | 
	#            |           *     |
	#            |   Kd  R         |
	#            |  | /     |      |
	#        +   |  |    +  |   -  |
	#      |IpG| |  |  |RpT|| |SmC||
	print("(i+g)*(l-(k/(r+t))*(s-c))")
	baumIpG= BinaryTree("+", BinaryTree("i"), BinaryTree("g"))
	baumRpT= BinaryTree("+", BinaryTree("r"), BinaryTree("r"))
	baumKdR= BinaryTree("/", BinaryTree("k"), baumRpT)
	baumSmC= BinaryTree("-", BinaryTree("s"), BinaryTree("c"))	
	baumKmS= BinaryTree("*", baumKdR, baumSmC)
	baumLmK= BinaryTree("-", BinaryTree("l"), baumKmS)	
	baumImL= BinaryTree("*", baumIpG, baumLmK)	
	print(baumImL)
	
	
	
	
